'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _https = require('https');

var _https2 = _interopRequireDefault(_https);

var _querystring = require('querystring');

var _querystring2 = _interopRequireDefault(_querystring);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/**
 * Checks if str is a valid JSON string
 *
 * @param {String} str Input string that should be a valid JSON
 * @returns Boolean
 */
var isJson = function isJson(str) {
  try {
    JSON.parse(str);
  } catch (e) {
    return false;
  }
  return true;
};

/**
 * Makes an http request
 *
 * @param {String} opts.protocol HTTP or HTTPS
 * @param {Object} opts Request connection options
 * @param {String} opts.method HTTP verb or method
 * @param {String} opts.host Host address
 * @param {String} opts.path URL path
 * @param {String} opts.port Server port
 * @param {Object} opts.headers HTTP headers
 * @param {Object} data Object to send or params in case of GET method
 * @returns Promise
 */
var request = function request(opts) {
  var data = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : null;
  return new Promise(function (resolve, reject) {
    try {
      var _opts = JSON.parse(JSON.stringify(opts));
      if (_opts.method.toUpperCase() === 'GET' && data) {
        _opts = Object.assign({}, _opts, { path: _opts.path + '?' + _querystring2.default.stringify(data) });
      }
      var req = _https2.default.request(_opts, function (res) {
        var str = '';
        res.on('data', function (chunk) {
          str += chunk;
        });
        res.on('end', function () {
          resolve(isJson(str) ? JSON.parse(str) : str);
        });
        res.on('error', function (e) {
          reject(e);
        });
      });
      if ((_opts.method.toUpperCase() === 'PUT' || _opts.method.toUpperCase() === 'POST') && data) {
        if (_opts.headers && (_opts.headers['Content-Type'] === 'application/json' || _opts.headers['content-type'] === 'application/json')) {
          req.write(JSON.stringify(data));
        } else {
          req.write(_querystring2.default.stringify(data));
        }
      }
      req.end();
    } catch (err) {
      reject(err);
    }
  });
};

exports.default = request;